"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
const request = require('request');
const inversify_1 = require('inversify');
const path = require('path');
const fs = require('fs');
const nodeUrl = require('url');
const _ = require('lodash');
const force_error_impl_1 = require("./force-error-impl");
const fileExists = require('file-exists');
const safeJsonParse = require('safe-json-parse/callback');
const async = require('async');
class RemoteCatalogGetterImpl extends force_error_impl_1.ForceErrorImpl {
    constructor(commandUtil) {
        super();
        this.commandUtil = commandUtil;
    }
    getCatalogFromUrl(url, cb) {
        let me = this;
        let parsedUrl = RemoteCatalogGetterImpl.getParsedUrl(url);
        if (parsedUrl.protocol) {
            me.baseUrl = `${parsedUrl.protocol}//${parsedUrl.hostname}${path.dirname(parsedUrl.path)}`;
        }
        else {
            me.baseUrl = path.dirname(parsedUrl.path);
        }
        me.getRemoteResource(parsedUrl, 'root', (err, remoteCatalogResource) => {
            if (me.commandUtil.callbackIfError(cb, err)) {
                return;
            }
            try {
                let remoteCatalog = remoteCatalogResource.parsedObject;
                let fnArray = [];
                remoteCatalog.entries.forEach(entry => {
                    entry.urls.forEach(url => {
                        let parsedUrl = RemoteCatalogGetterImpl.getParsedUrl(url);
                        if (!parsedUrl.protocol) {
                            url = path.isAbsolute(parsedUrl.path) ? parsedUrl.path : `${me.baseUrl}/${parsedUrl.path}`;
                        }
                        fnArray.push(async.apply(me.getRemoteResource.bind(me), url, entry.name));
                    });
                });
                async.parallel(fnArray, (err, results) => {
                    remoteCatalog.entries.forEach(entry => {
                        entry.resources =
                            _.filter(results, ['parentCatalogEntryName', entry.name]);
                    });
                    cb(null, remoteCatalog);
                });
            }
            catch (err) {
                me.commandUtil.callbackIfError(cb, err);
            }
        });
    }
    getRemoteResource(url, parentCatalogEntryName, cb) {
        let me = this;
        cb = me.checkCallback(cb);
        me.resolveTextResourceFromUrl(url, (err, text, absoluteUrl) => {
            safeJsonParse(text, (err, parsedObject) => {
                let name = path.basename(absoluteUrl);
                cb(null, {
                    absoluteUrl, name, text, parsedObject, parentCatalogEntryName
                });
            });
        });
    }
    resolveJsonObjectFromUrl(url, cb) {
        let me = this;
        cb = cb || (() => {
        });
        me.resolveTextResourceFromUrl(url, (err, text, absoluteUrl) => {
            safeJsonParse(text, (err, jsonObject) => {
                cb(err, jsonObject, absoluteUrl);
            });
        });
    }
    resolveTextResourceFromUrl(url, cb) {
        let me = this;
        cb = me.checkCallback(cb);
        try {
            let parsedUrl = RemoteCatalogGetterImpl.getParsedUrl(url);
            if (!parsedUrl) {
                cb(new Error(`Url: '${url}' is unusable`));
                return;
            }
            if (!parsedUrl.protocol) {
                let urlString = parsedUrl.path;
                if (!fileExists(urlString)) {
                    cb(new Error(`${urlString} doesn't exist`), null);
                    return;
                }
                cb(null, fs.readFileSync(urlString, 'utf8'), urlString);
                return;
            }
            request(parsedUrl.href, (err, res, text) => {
                if (res.statusCode !== 200) {
                    cb(new Error(`Error retrieving '${parsedUrl.href}'`), null);
                    return;
                }
                cb(null, text, parsedUrl.href);
            });
        }
        catch (err) {
            cb(err, null);
        }
    }
    static getParsedUrl(url) {
        try {
            if (url.constructor.name === 'Url') {
                return url;
            }
            else if (url.constructor.name === 'String') {
                return nodeUrl.parse(url);
            }
        }
        catch (err) {
            return null;
        }
    }
}
RemoteCatalogGetterImpl = __decorate([
    __param(0, inversify_1.inject('CommandUtil')), 
    __metadata('design:paramtypes', [Object])
], RemoteCatalogGetterImpl);
exports.RemoteCatalogGetterImpl = RemoteCatalogGetterImpl;
//# sourceMappingURL=remote-catalog-impl.js.map