"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
const inversify_1 = require("inversify");
const child_process_1 = require('child_process');
const force_error_impl_1 = require("./force-error-impl");
const readlineSync = require('readline-sync');
const inpathSync = require('inpath').sync;
const pidof = require('pidof');
const psTree = require('ps-tree');
let SpawnImpl_1 = class SpawnImpl extends force_error_impl_1.ForceErrorImpl {
    constructor(commandUtil) {
        super();
        this.commandUtil = commandUtil;
    }
    spawnShellCommandPipelineAsync(cmdArray, optionsArray, cbStatusOrFinal, cbFinal) {
        let lclCmdArray = cmdArray.slice(0);
        let lclOptionsArray = optionsArray.slice(0);
        let cmd = lclCmdArray.pop().slice(0);
        let childProcess = this.spawnShellCommandAsync(cmd, lclOptionsArray.pop(), cbStatusOrFinal, cbFinal);
        let proc = childProcess;
        while (cmd = lclCmdArray.pop()) {
            cmd = cmd.slice(0);
            let prevStdin = proc.stdin;
            proc = this.spawnShellCommandAsync(cmd, lclOptionsArray.pop(), cbStatusOrFinal, (err, results) => {
                process.stderr.write(results);
            });
            proc.stdout.pipe(prevStdin);
        }
        return childProcess;
    }
    sudoSpawnPipelineAsync(cmdArray, optionsArray, cbStatusOrFinal, cbFinal) {
        return undefined;
    }
    validate_spawnShellCommandAsync_args(cmd, options = null, cbStatus = null, cbFinal = null) {
        cmd = cmd || [];
        cmd = cmd.slice(0);
        if (typeof cbFinal !== 'function') {
            if (typeof cbStatus !== 'function') {
                cbFinal = cbStatus = () => {
                };
            }
            else {
                cbFinal = cbStatus;
                cbStatus = () => {
                };
            }
        }
        else {
            if (typeof cbStatus !== 'function') {
                cbStatus = () => {
                };
            }
        }
        options = options || {};
        options.preSpawnMessage = options.preSpawnMessage || '';
        options.postSpawnMessage = options.postSpawnMessage || '';
        options.showDiagnostics = options.showDiagnostics || false;
        options.suppressStdErr = options.suppressStdErr || false;
        options.suppressStdOut = options.suppressStdOut || false;
        options.cacheStdErr = options.cacheStdErr || false;
        options.cacheStdOut = options.cacheStdOut || false;
        options.suppressFinalStats = options.suppressFinalStats || false;
        options.stdio = options.stdio || 'pipe';
        options.cwd = options.cwd || process.cwd();
        return { cmd, options, cbStatus, cbFinal };
    }
    spawnShellCommandAsync(_cmd, _options = null, _cbStatus = null, _cbFinal = null) {
        let me = this;
        let { cmd, options, cbStatus, cbFinal } = me.validate_spawnShellCommandAsync_args(_cmd, _options, _cbStatus, _cbFinal);
        if (me.checkForceError('spawnShellCommandAsync', cbFinal)) {
            return;
        }
        let childProcess;
        try {
            let command = cmd[0];
            let args = cmd.slice(1);
            let stdoutText = '';
            let stderrText = '';
            me.diagnosticTrace(`Running '${cmd}' @ '${options.cwd}'`, options);
            SpawnImpl_1.reportStatus(cbStatus, options.preSpawnMessage);
            childProcess = child_process_1.spawn(command, args, options);
            childProcess.stderr.on('data', (dataChunk) => {
                if (options.suppressStdErr && !options.cacheStdErr) {
                    return;
                }
                let text = dataChunk.toString();
                if (!options.suppressStdErr) {
                    cbStatus(new Error(text), text);
                }
                if (options.cacheStdErr) {
                    stderrText += text;
                }
            });
            childProcess.stdout.on('data', (dataChunk) => {
                if (options.suppressStdOut && !options.cacheStdOut) {
                    return;
                }
                let text = dataChunk.toString();
                if (!options.suppressStdOut) {
                    cbStatus(null, text);
                }
                if (options.cacheStdOut) {
                    stdoutText += text;
                }
            });
            childProcess.on('error', (code) => {
                cbFinal = SpawnImpl_1.childCloseOrExit(code, '', stdoutText, stderrText, options, cbStatus, cbFinal);
            });
            childProcess.on('exit', (code, signal) => {
                cbFinal = SpawnImpl_1.childCloseOrExit(code, signal, stdoutText, stderrText, options, cbStatus, cbFinal);
            });
            childProcess.on('close', (code, signal) => {
                cbFinal = SpawnImpl_1.childCloseOrExit(code, signal, stdoutText, stderrText, options, cbStatus, cbFinal);
            });
        }
        catch (err) {
            cbFinal(err, null);
        }
        return childProcess;
    }
    static reportStatus(cbStatus, status) {
        if (!cbStatus || !status) {
            return;
        }
        cbStatus(null, status);
    }
    diagnosticTrace(msg, options) {
        if (!options.showDiagnostics) {
            return;
        }
        this.commandUtil.log(msg);
    }
    static childCloseOrExit(code, signal, stdoutText, stderrText, options, cbStatus, cbFinal) {
        if (cbFinal) {
            SpawnImpl_1.reportStatus(cbStatus, options.postSpawnMessage);
            let returnString = !options.suppressFinalStats
                ? JSON.stringify({ code, signal, stdoutText, stderrText }, undefined, 2)
                : '';
            let error = (code !== null && code !== 0)
                ? new Error(returnString)
                : null;
            cbFinal(error, returnString);
        }
        return null;
    }
    sudoSpawnAsync(cmd, options = null, cbStatusOrFinal = null, cbFinal = null) {
        let me = this;
        let prompt = '#node-sudo-passwd#';
        let prompts = 0;
        let args = ['-S', '-p', prompt];
        cmd = cmd || [];
        cmd = cmd.slice(0);
        [].push.apply(args, cmd);
        let path = process.env['PATH'].split(':');
        let sudoBin = inpathSync('sudo', path);
        args.unshift(sudoBin);
        let child = me.spawnShellCommandAsync(args, options, cbStatusOrFinal, cbFinal);
        if (!child) {
            return;
        }
        function waitForStartup(err, children) {
            if (err) {
                throw new Error(`Error spawning process`);
            }
            if (children && children.length) {
                child.stderr.removeAllListeners();
            }
            else {
                setTimeout(function () {
                    psTree(child.pid, waitForStartup);
                }, 100);
            }
        }
        psTree(child.pid, waitForStartup);
        child.stderr.on('data', function (data) {
            let lines = data.toString().trim().split('\n');
            lines.forEach(function (line) {
                if (line === prompt) {
                    if (++prompts > 1) {
                        me.cachedPassword = null;
                    }
                    let username = require('username').sync();
                    let loginMessage = (prompts > 1)
                        ? `Sorry, try again.\n[sudo] password for ${username}: `
                        : `[sudo] password for ${username}: `;
                    if (me.cachedPassword) {
                        child.stdin.write(me.cachedPassword + '\n');
                    }
                    else {
                        me.cachedPassword = readlineSync.question(loginMessage, { hideEchoBack: true });
                        child.stdin.write(me.cachedPassword + '\n');
                    }
                }
            });
        });
        return child;
    }
};
let SpawnImpl = SpawnImpl_1;
SpawnImpl = SpawnImpl_1 = __decorate([
    inversify_1.injectable(),
    __param(0, inversify_1.inject('CommandUtil')), 
    __metadata('design:paramtypes', [Object])
], SpawnImpl);
exports.SpawnImpl = SpawnImpl;
//# sourceMappingURL=spawn-impl.js.map